Package: ngme2
Type: Package
Title: Linear Latent Non-Gaussian Models with Flexible Distributions
Version: 0.9.8
Authors@R: c(
    person("David", "Bolin", email = "davidbolin@gmail.com", role = c("aut", "cph")),
    person("Xiaotian", "Jin", email = "xiaotian.jin@kaust.edu.sa", role =c("aut", "cre")),
    person("Alexandre", "Simas", email = "alexandre.impa@gmail.com", role = "aut"),
    person("Jonas", "Wallin", email = "jonas.wallin81@gmail.com", role = "aut"),
    person("Andrea V.", "Rocha", email = "dearocha@yahoo.com", role = "ctb", comment = "contributed to many vignettes and examples"),
    person("Timothy A.", "Davis", role = c("ctb", "cph"), comment = "SuiteSparse components"),
    person("Patrick R.", "Amestoy", role = c("ctb", "cph"), comment = "SuiteSparse AMD and CAMD components"),
    person("Iain S.", "Duff", role = c("ctb", "cph"), comment = "SuiteSparse AMD and CAMD components"),
    person("John K.", "Reid", role = c("ctb", "cph"), comment = "SuiteSparse AMD-derived code"),
    person("Yanqing", "Chen", role = c("ctb", "cph"), comment = "SuiteSparse CAMD components"),
    person("Sivasankaran", "Rajamanickam", role = c("ctb", "cph"), comment = "SuiteSparse CCOLAMD components"),
    person("Stefan", "Larimore", role = c("ctb", "cph"), comment = "SuiteSparse CCOLAMD and COLAMD components"),
    person("William W.", "Hager", role = c("ctb", "cph"), comment = "SuiteSparse CHOLMOD Modify components"),
    person("University of Florida", role = "cph", comment = "SuiteSparse CHOLMOD and CCOLAMD components"),
    person("Regents of the University of Minnesota", role = "cph", comment = "METIS/GKlib components bundled with SuiteSparse"),
    person("Free Software Foundation, Inc.", role = "cph", comment = "GKlib regex/getopt code bundled with SuiteSparse"),
    person("Makoto", "Matsumoto", role = c("ctb", "cph"), comment = "Mersenne Twister code bundled with SuiteSparse"),
    person("Takuji", "Nishimura", role = c("ctb", "cph"), comment = "Mersenne Twister code bundled with SuiteSparse"),
    person("Alexander", "Chemeris", role = c("ctb", "cph"), comment = "MS integer headers bundled with SuiteSparse"))
Maintainer: Xiaotian Jin <xiaotian.jin@kaust.edu.sa>
Description: Fits and analyzes linear latent non-Gaussian models for
    temporal, spatial, and space-time data. The package provides model
    components for autoregressive and Ornstein-Uhlenbeck processes, random
    walks, Matern fields based on stochastic partial differential equations,
    separable and non-separable space-time models, graph-based Matern models,
    bivariate type-G fields, and user-defined sparse operators. Latent fields
    and observation models can use Gaussian and non-Gaussian noise
    distributions, including normal inverse Gaussian, generalized asymmetric
    Laplace, and skew-t distributions. Functions are included for simulation,
    likelihood-based estimation, prediction, cross-validation, convergence
    diagnostics, stochastic gradient optimization, batch-means confidence
    intervals, and posterior-like sampling. The modeling framework is described
    in Bolin, Jin, Simas and Wallin (2026) "A Unified and Computationally
    Efficient Non-Gaussian Statistical Modeling Framework"
    <doi:10.48550/arXiv.2602.23987>.
License: GPL (>= 3)
Encoding: UTF-8
LazyData: true
Imports: Matrix, Rcpp, methods, rlang, ggplot2, graphics, fmesher,
        stats, gridExtra, withr
LinkingTo: Rcpp, RcppEigen,
Suggests: R.rsp, knitr, rmarkdown, testthat (>= 3.0.0), MASS, dplyr,
        fields, inlabru, MetricGraph, rSPDE, sf, zoo, mvtnorm
VignetteBuilder: knitr
RoxygenNote: 7.3.2
Config/testthat/edition: 3
URL: https://davidbolin.github.io/ngme2/,
        https://github.com/davidbolin/ngme2
Depends: R (>= 3.5.0)
BugReports: https://github.com/davidbolin/ngme2/issues
Copyright: The R package and code, and the main programs, were written
        by and are Copyright by David Bolin, Xiaotian Jin, Alexandre
        Simas, and Jonas Wallin, and are redistributable under the GNU
        Public License, version 3 or later. The package also includes
        bundled SuiteSparse components. For details see
        inst/COPYRIGHTS.
NeedsCompilation: yes
Packaged: 2026-05-19 19:01:57 UTC; jinx0a
Author: David Bolin [aut, cph],
  Xiaotian Jin [aut, cre],
  Alexandre Simas [aut],
  Jonas Wallin [aut],
  Andrea V. Rocha [ctb] (contributed to many vignettes and examples),
  Timothy A. Davis [ctb, cph] (SuiteSparse components),
  Patrick R. Amestoy [ctb, cph] (SuiteSparse AMD and CAMD components),
  Iain S. Duff [ctb, cph] (SuiteSparse AMD and CAMD components),
  John K. Reid [ctb, cph] (SuiteSparse AMD-derived code),
  Yanqing Chen [ctb, cph] (SuiteSparse CAMD components),
  Sivasankaran Rajamanickam [ctb, cph] (SuiteSparse CCOLAMD components),
  Stefan Larimore [ctb, cph] (SuiteSparse CCOLAMD and COLAMD components),
  William W. Hager [ctb, cph] (SuiteSparse CHOLMOD Modify components),
  University of Florida [cph] (SuiteSparse CHOLMOD and CCOLAMD
    components),
  Regents of the University of Minnesota [cph] (METIS/GKlib components
    bundled with SuiteSparse),
  Free Software Foundation, Inc. [cph] (GKlib regex/getopt code bundled
    with SuiteSparse),
  Makoto Matsumoto [ctb, cph] (Mersenne Twister code bundled with
    SuiteSparse),
  Takuji Nishimura [ctb, cph] (Mersenne Twister code bundled with
    SuiteSparse),
  Alexander Chemeris [ctb, cph] (MS integer headers bundled with
    SuiteSparse)
Repository: CRAN
Date/Publication: 2026-05-20 07:40:02 UTC
Built: R 4.5.2; x86_64-apple-darwin20; 2026-05-25 01:54:43 UTC; unix
Archs: ngme2.so.dSYM
