# Generated by roxygen2: do not edit by hand

S3method(noise_moments,ngme_noise)
S3method(plot,ngme_noise)
S3method(plot,ngme_sgld_ci)
S3method(plot,parameter_distance)
S3method(predict,ngme)
S3method(print,ngme)
S3method(print,ngme_model)
S3method(print,ngme_noise)
S3method(print,ngme_operator)
S3method(print,ngme_replicate)
S3method(print,ngme_trajectories)
S3method(print,noise_kld_comparison)
S3method(print,parameter_distance)
S3method(print,summary.ngme_batch_ci)
S3method(simulate,ngme)
S3method(simulate,ngme_model)
S3method(simulate,ngme_noise)
S3method(summary,ngme)
S3method(summary,ngme_batch_ci)
export(adagrad)
export(adam)
export(adamW)
export(adaptive_gd)
export(ar)
export(ar1)
export(arma)
export(arma11)
export(batch_decay)
export(batch_means_ci)
export(batch_means_estimator)
export(bfgs)
export(bv)
export(bv_matern)
export(calibrate_inv_exp_lambda)
export(calibrate_inv_exp_lambda_driven_nig)
export(compare_noise_kld)
export(compute_index_corr_from_map)
export(compute_log_like)
export(compute_ngme_CI)
export(compute_ngme_ci)
export(compute_ngme_sgld_samples)
export(control_ngme)
export(control_opt)
export(control_opt_batch_ci)
export(create_paired_cv_splits)
export(cross_validation)
export(dgal)
export(dgig)
export(dig)
export(digam)
export(dnig)
export(f)
export(generic)
export(generic_ns)
export(get_data_from_formula)
export(get_parameter_distance)
export(get_trace_trajectories)
export(get_trajectories)
export(iid)
export(make_time_series_cv_index)
export(matern)
export(mean_list)
export(merge_noise)
export(momentum)
export(name2fun)
export(ngme)
export(ngme_as_sparse)
export(ngme_batch_ci)
export(ngme_cov_matrix)
export(ngme_make_mesh_repls)
export(ngme_model_types)
export(ngme_noise_types)
export(ngme_optimizers)
export(ngme_post_samples)
export(ngme_prior_types)
export(ngme_result)
export(ngme_sgld_ci)
export(ngme_sgld_samples)
export(ngme_ts_make_A)
export(ngme_update)
export(noise_gal)
export(noise_gal_moments)
export(noise_moments)
export(noise_nig)
export(noise_nig_moments)
export(noise_normal)
export(noise_normal_nig)
export(noise_skew_t)
export(noise_t)
export(openmp_test)
export(ou)
export(pgal)
export(pgig)
export(pig)
export(pigam)
export(pnig)
export(poly_decay)
export(posterior_plot)
export(precision_matrix_multivariate)
export(precision_matrix_multivariate_spde)
export(precond_sgd)
export(prior_half_cauchy)
export(prior_inv_exp)
export(prior_inv_exponential)
export(prior_none)
export(prior_normal)
export(prior_pc_sd)
export(priors)
export(qgal)
export(qgig)
export(qig)
export(qigam)
export(qnig)
export(re)
export(rgal)
export(rgig)
export(rig)
export(rigam)
export(rmsprop)
export(rnig)
export(rw1)
export(rw2)
export(sgd)
export(sgld)
export(spacetime)
export(stepsize_control)
export(stepsize_decay)
export(stepsize_schedule)
export(tp)
export(traceplot)
export(var1)
import(Rcpp)
importFrom(Rcpp,evalCpp)
importFrom(fmesher,fm_basis)
importFrom(fmesher,fm_fem)
importFrom(fmesher,fm_mesh_1d)
importFrom(fmesher,fm_mesh_2d)
importFrom(ggplot2,aes)
importFrom(ggplot2,geom_hline)
importFrom(ggplot2,geom_line)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,guides)
importFrom(ggplot2,labs)
importFrom(ggplot2,xlab)
importFrom(ggplot2,ylab)
importFrom(graphics,hist)
importFrom(methods,as)
importFrom(rlang,.data)
importFrom(stats,aggregate)
importFrom(stats,as.formula)
importFrom(stats,delete.response)
importFrom(stats,density)
importFrom(stats,dgamma)
importFrom(stats,dist)
importFrom(stats,dnorm)
importFrom(stats,formula)
importFrom(stats,median)
importFrom(stats,model.frame)
importFrom(stats,model.matrix)
importFrom(stats,qgamma)
importFrom(stats,qlogis)
importFrom(stats,quantile)
importFrom(stats,rgamma)
importFrom(stats,rnorm)
importFrom(stats,sd)
importFrom(stats,setNames)
importFrom(stats,simulate)
importFrom(stats,terms)
importFrom(stats,terms.formula)
importFrom(utils,capture.output)
importFrom(utils,head)
importFrom(utils,modifyList)
importFrom(utils,str)
importFrom(utils,tail)
useDynLib(ngme2, .registration = TRUE)
