# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

estimate_cpp <- function(R_ngme, control_opt) {
    .Call(`_ngme2_estimate_cpp`, R_ngme, control_opt)
}

sampling_cpp <- function(ngme_replicate, n, n_burnin, posterior, seed) {
    .Call(`_ngme2_sampling_cpp`, ngme_replicate, n, n_burnin, posterior, seed)
}

get_openmp_threads <- function() {
    .Call(`_ngme2_get_openmp_threads`)
}

has_pardiso <- function() {
    .Call(`_ngme2_has_pardiso`)
}

compute_log_like_cpp <- function(R_ngme) {
    .Call(`_ngme2_compute_log_like_cpp`, R_ngme)
}

rGIG_cpp <- function(p, a, b, seed) {
    .Call(`_ngme2_rGIG_cpp`, p, a, b, seed)
}

