% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/optimizer.R
\name{adagrad}
\alias{adagrad}
\title{AdaGrad SGD optimization}
\usage{
adagrad(stepsize = 0.05, epsilon = 1e-08)
}
\arguments{
\item{stepsize}{stepsize for SGD}

\item{epsilon}{epsilon for numerical stability}
}
\value{
a list of control variables for optimization
(used in \code{control_opt} function)
}
\description{
AdaGrad SGD optimization
}
\details{
The update rule for AdaGrad is:
\deqn{v_t = v_{t-1} + g_t^2}
\deqn{x_{t+1} = x_t - \text{stepsize} * \frac{g_t}{\sqrt{v_t} + \epsilon}}
}
