% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/batch-means.R
\name{summary.ngme_batch_ci}
\alias{summary.ngme_batch_ci}
\title{Summary for Batch-Means CI Results}
\usage{
\method{summary}{ngme_batch_ci}(object, digits = 4, ...)
}
\arguments{
\item{object}{an object of class `ngme_batch_ci`.}

\item{digits}{number of digits used for printing.}

\item{...}{currently unused.}
}
\value{
A list with summary table and covariance matrix.
}
\description{
Summarize an object of class `ngme_batch_ci`, including parameter means,
standard errors, confidence intervals, and covariance matrix.
}
