% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/models.R
\name{matern}
\alias{matern}
\title{ngme Matern SPDE model specification}
\usage{
matern(
  mesh = NULL,
  alpha = 2,
  fix_alpha = TRUE,
  kappa = NULL,
  theta_kappa = NULL,
  rational_order = 2,
  B_kappa = NULL
)
}
\arguments{
\item{mesh}{an fmesher::fm_mesh_2d object, mesh for build the SPDE model}

\item{alpha}{SPDE smoothness parameter (alpha = 2beta) 2 or 4 for integer case, otherwise for fractional case}

\item{fix_alpha}{if FALSE, enable fractional modeling}

\item{kappa}{the range parameter, for the stationary model, it is the range parameter}

\item{theta_kappa}{kappa = exp(B_kappa * theta_kappa), for the non-stationary model, it is the range parameter}

\item{rational_order}{order of rational approximation for fractional operators (default: 2)}

\item{B_kappa}{bases for theta_kappa, ignore if use the stationary model}
}
\value{
ngme_operator object
}
\description{
ngme Matern SPDE model specification
}
