% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/optimizer.R
\name{sgd}
\alias{sgd}
\title{Vanilla SGD optimization}
\usage{
sgd(stepsize = 0.001)
}
\arguments{
\item{stepsize}{stepsize for SGD}
}
\value{
a list of control variables for optimization
(used in \code{control_opt} function)
}
\description{
Simple stochastic gradient descent without momentum or preconditioning.
}
