% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/prior.R
\name{priors}
\alias{priors}
\title{Prior Container}
\usage{
priors(...)
}
\arguments{
\item{...}{named prior specifications}
}
\value{
named prior container
}
\description{
Prior Container
}
