% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/validation.R
\name{compute_score_given_pred}
\alias{compute_score_given_pred}
\title{Compute the scores given the prediction}
\usage{
compute_score_given_pred(
  Y_N_1_thin,
  Y_N_2_thin,
  y_data,
  group_data,
  merge_groups = FALSE,
  merged_group_name = NULL,
  metric = NULL
)
}
\arguments{
\item{Y_N_1_thin}{posterior predictive draws (rows = observations, columns = samples)}

\item{Y_N_2_thin}{posterior predictive draws (rows = observations, columns = samples)}

\item{y_data}{a vector of length n_obs}

\item{group_data}{a vector of length n_obs}

\item{merge_groups}{logical, if TRUE and there are exactly two groups,
combine them into a vector-valued score}

\item{merged_group_name}{optional label for the merged group}

\item{metric}{optional custom metric function used to combine group-wise values before scoring}
}
\description{
Compute the scores given the prediction
}
