% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.ngme.R
\name{print.ngme_trajectories}
\alias{print.ngme_trajectories}
\title{Print method for ngme_trajectories}
\usage{
\method{print}{ngme_trajectories}(x, ...)
}
\arguments{
\item{x}{ngme_trajectories object}

\item{...}{additional arguments}
}
\value{
Invisibly returns \code{x}, an \code{ngme_trajectories} object. The
  method is called for its side effect of printing a summary of stored
  parameter trajectory matrices, including parameter names, chain count, and
  iteration count.
}
\description{
Print method for ngme_trajectories
}
