% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.ngme.R
\name{print.parameter_distance}
\alias{print.parameter_distance}
\title{Print method for parameter_distance}
\usage{
\method{print}{parameter_distance}(x, ...)
}
\arguments{
\item{x}{parameter_distance object}

\item{...}{additional arguments}
}
\value{
Invisibly returns \code{x}, a numeric vector with class
  \code{parameter_distance}. The method is called for its side effect of
  printing the norm type, chain summary, number of iterations, final distance,
  and true parameter values.
}
\description{
Print method for parameter_distance
}
