% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ngme.constant.R
\name{ngme_model_types}
\alias{ngme_model_types}
\title{Show ngme model types}
\usage{
ngme_model_types()
}
\value{
available types for models
}
\description{
Show ngme model types
}
