% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/noise.R
\name{print.ngme_noise}
\alias{print.ngme_noise}
\title{Print ngme noise}
\usage{
\method{print}{ngme_noise}(x, padding = 0, prefix = "Noise type", model_type = NULL, ...)
}
\arguments{
\item{x}{noise object}

\item{padding}{number of white space padding in front}

\item{prefix}{prefix}

\item{model_type}{model type}

\item{...}{...}
}
\value{
a list (noise specifications)
}
\description{
Print ngme noise
}
