% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/models.R
\name{ARcov}
\alias{ARcov}
\title{Helper function to compute AR(p) autocovariance}
\usage{
ARcov(rho, p)
}
\arguments{
\item{rho}{vector of AR coefficients}

\item{p}{AR order}
}
\value{
vector of autocovariances from lag 0 to lag p-1
}
\description{
Helper function to compute AR(p) autocovariance
}
\keyword{internal}
