% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/optimizer.R
\name{ngme_optimizers}
\alias{ngme_optimizers}
\title{List supported optimizers}
\usage{
ngme_optimizers()
}
\value{
a character vector of supported optimizers
}
\description{
This function returns a list of supported optimizers in the \code{ngme} package.
The optimizers are categorized into three groups:
\itemize{
\item \strong{Gradient descent}: \code{"sgd"}, \code{"sgld"}, \code{"momentum"}, \code{"adaptive_gd"}
\item \strong{Adaptive learning rate}: \code{"adagrad"}, \code{"rmsprop"}, \code{"adam"}, \code{"adamW"}
\item \strong{Preconditioner}: \code{"precond_sgd"}, \code{"bfgs"}
}
}
