% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/models.R
\name{re}
\alias{re}
\title{ngme random effect model}
\usage{
re(map, theta_K = NULL, ...)
}
\arguments{
\item{map}{numerical vector, covariates to build index for the process (can be formula, provided data)}

\item{theta_K}{initial value for theta_K (build covariance matrix)}

\item{...}{currently ignored}
}
\value{
ngme_operator object
}
\description{
ngme random effect model
}
