% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/prior.R
\name{prior_inv_exponential}
\alias{prior_inv_exponential}
\alias{prior_inv_exp}
\title{Prior Inverse-Exponential}
\usage{
prior_inv_exponential(lambda = 1, lower = 0, target = "coef")

prior_inv_exp(lambda = 1, lower = 0, target = "coef")
}
\arguments{
\item{lambda}{exponential rate on \eqn{\kappa = 1/\nu}}

\item{lower}{lower shift used in \eqn{\nu = \mathrm{lower} + \exp(\theta)}}

\item{target}{apply prior on coefficient scale (`"coef"`) or field scale (`"field"`)}
}
\value{
prior specification
}
\description{
Prior induced by \eqn{\kappa = 1 / \nu \sim \mathrm{Exp}(\lambda)},
giving \eqn{p(\nu) = \lambda \exp(-\lambda / \nu)\nu^{-2}} for \eqn{\nu > 0}.
Internally this prior is applied to \eqn{\nu = \mathrm{lower} + \exp(\theta)}.
}
