% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/optimizer.R
\name{momentum}
\alias{momentum}
\title{Momentum SGD optimization}
\usage{
momentum(stepsize = 0.05, beta1 = 0.9, beta2 = 1 - beta1)
}
\arguments{
\item{stepsize}{stepsize for SGD}

\item{beta1}{beta1 for momentum}

\item{beta2}{beta2 for momentum}
}
\value{
a list of control variables for optimization
(used in \code{control_opt} function)
}
\description{
Momentum SGD optimization
}
\details{
The update rule for momentum is:
\deqn{v_t = \beta_1 v_{t-1} + \beta_2 g_t}
\deqn{x_{t+1} = x_t - \text{stepsize} * v_t}
}
