% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/simulation.R
\name{simulate.ngme}
\alias{simulate.ngme}
\title{Simulate from a ngme object (possibly with replicates)}
\usage{
\method{simulate}{ngme}(object, nsim = 1, seed = NULL, ...)
}
\arguments{
\item{object}{ngme object}

\item{nsim}{number of simulations}

\item{seed}{seed}

\item{...}{optional arguments. Supported names are \code{posterior}
(whether to simulate from posterior sampling of latent fields) and
\code{m_noise} (whether to add the measurement noise).}
}
\value{
a realization of ngme object
}
\description{
Simulate from a ngme object (possibly with replicates)
}
