% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/info_datasets.R
\docType{data}
\name{cienaga}
\alias{cienaga}
\title{The swamp of Cienaga Grande in Santa Marta, Colombia}
\format{
A data frame with 218 rows and 6 columns.
\describe{
  \item{East, North}{location}
  \item{depth}{depth of the swamp}
  \item{temp}{temperature}
  \item{oxyg}{oxygen}
  \item{measurement}{1 means the first measurement, 2 the second}
}
}
\source{
..
}
\usage{
cienaga
}
\description{
There is a total of 114 locations where some properties of the
swamp were measured. Those measurements were taken twice, however there is no information available about their chronological order so this data cannot
be treated as spatiotemporal, despite that, the multiple measurements can be treated as replicates.
}
\keyword{datasets}
