% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/prior.R
\name{prior_none}
\alias{prior_none}
\title{Prior None}
\usage{
prior_none(target = "coef")
}
\arguments{
\item{target}{apply prior on coefficient scale (`"coef"`) or field scale (`"field"`)}
}
\value{
prior specification
}
\description{
Prior None
}
