% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/validation.R
\name{merge_replicates}
\alias{merge_replicates}
\title{Merge model of replicates into model of 1 replicate given train_idx and
test_idx, the merged model contains all the information of train_idx from
different replicates.}
\usage{
merge_replicates(ngme, train_idx, test_idx)
}
\arguments{
\item{ngme}{a ngme object}

\item{train_idx}{a vector of indices of train data}

\item{test_idx}{a vector of indices of test data}
}
\description{
Merge model of replicates into model of 1 replicate given train_idx and
test_idx, the merged model contains all the information of train_idx from
different replicates.
}
