% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/batch-means.R
\name{ngme_sgld_ci}
\alias{ngme_sgld_ci}
\title{Quantile Confidence Intervals from SGLD Samples}
\usage{
ngme_sgld_ci(samples, lower = 0.025, upper = 0.975)
}
\arguments{
\item{samples}{data.frame (or list of data.frames) returned by
[ngme_sgld_samples()].}

\item{lower}{lower quantile probability (e.g. 0.025).}

\item{upper}{upper quantile probability (e.g. 0.975).}
}
\value{
A list with posterior-like point estimates (`estimates`), quantile
intervals (`ci`), and sample covariance matrix (`covariance`).
Class: `ngme_sgld_ci`.
}
\description{
Compute parameter-wise quantile confidence intervals from posterior-like
SGLD samples returned by [ngme_sgld_samples()].
}
