% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ngme.constant.R
\name{ngme_prior_types}
\alias{ngme_prior_types}
\title{Show ngme priors}
\usage{
ngme_prior_types()
}
\value{
available types of priors
}
\description{
Show ngme priors
}
