% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/optimizer.R
\name{bfgs}
\alias{bfgs}
\title{BFGS optimization}
\usage{
bfgs(line_search = "wolfe")
}
\arguments{
\item{line_search}{line search method, can be c("backtracking", "wofle")
"bfgs" means use BFGS preconditioner}
}
\value{
a list of control variables for optimization
(used in \code{control_opt} function)
}
\description{
BFGS optimization
}
