% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ngme.R
\name{ngme_parse_formula}
\alias{ngme_parse_formula}
\title{Parse the formula for ngme function}
\usage{
ngme_parse_formula(
  fm,
  data,
  control_ngme,
  noise,
  group,
  replicate,
  prior_beta,
  standardize
)
}
\arguments{
\item{fm}{formula}

\item{data}{data.frame}

\item{control_ngme}{control_ngme}

\item{noise}{noise}

\item{group}{group factor}

\item{replicate}{replicate vector}

\item{prior_beta}{prior specification for fixed effects}

\item{standardize}{logical, whether fixed effects are standardized}
}
\value{
a list (replicate) of ngme_replicate models
}
\description{
Parse the formula for ngme function
}
