% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/operator.R
\name{print.ngme_model}
\alias{print.ngme_model}
\title{Print ngme model}
\usage{
\method{print}{ngme_model}(x, padding = 0, ...)
}
\arguments{
\item{x}{ngme model object}

\item{padding}{number of white space padding in front}

\item{...}{...}
}
\value{
a list (model specifications)
}
\description{
Print ngme model
}
