% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/prior.R
\name{prior_normal}
\alias{prior_normal}
\title{Prior Normal}
\usage{
prior_normal(mean = 0, sd = 1, target = "coef")
}
\arguments{
\item{mean}{prior mean}

\item{sd}{prior standard deviation}

\item{target}{apply prior on coefficient scale (`"coef"`) or field scale (`"field"`)}
}
\value{
prior specification
}
\description{
Prior Normal
}
