% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.ngme.R
\name{traceplot}
\alias{traceplot}
\title{Trace plot of ngme fitting}
\usage{
traceplot(
  ngme,
  name = "all",
  moving_window = 1,
  hline = NULL,
  combine = TRUE,
  ncol = NULL
)
}
\arguments{
\item{ngme}{ngme object}

\item{name}{name of latent models, otherwise plot fixed effects and measurement noise.
  Use \code{"all"} to plot all latent models plus data parameters in one figure.
should be in names(ngme$models) or other}

\item{moving_window}{moving window for the traceplot}

\item{hline}{vector, add hline to each plot}

\item{combine}{bool, if TRUE, return a single faceted ggplot; otherwise return
a list of ggplot objects and print them one by one.}

\item{ncol}{number of facet columns when \code{combine = TRUE}. If \code{NULL}
(default), use 2 columns for multi-parameter plots and 1 for single-parameter plots.}
}
\value{
A ggplot object when \code{combine = TRUE}; otherwise a list of ggplot
  objects. The mean parameter trajectories are stored in the \code{avg_lines}
  attribute of the returned object.
}
\description{
Trace plot of ngme fitting
}
