% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ngme_replicate.R
\name{print.ngme_replicate}
\alias{print.ngme_replicate}
\title{Print ngme object}
\usage{
\method{print}{ngme_replicate}(x, ...)
}
\arguments{
\item{x}{ngme object}

\item{...}{ignored}
}
\value{
Invisibly returns \code{x}, an \code{ngme_replicate} object. The
  method is called for its side effect of printing a replicate summary.
}
\description{
Print ngme object
}
