% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/simulation.R
\name{simulate.ngme_model}
\alias{simulate.ngme_model}
\title{Simulate latent process with noise}
\usage{
\method{simulate}{ngme_model}(object, nsim = 1, seed = NULL, ...)
}
\arguments{
\item{object}{ngme model specified by f() function}

\item{nsim}{number of simulations}

\item{seed}{seed}

\item{...}{ignored}
}
\value{
a realization of latent model
}
\description{
Simulate latent process with noise
}
\examples{
simulate(f(1:10, model = ar1(rho = 0.4), noise = noise_nig()))
simulate(f(rnorm(10), model = rw1(), noise = noise_normal()))
simulate(f(1:10, model = ar1(rho = 0.4), noise = noise_t(nu = 5)))
}
