% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/complex-models.R
\name{tp}
\alias{tp}
\title{ngme tensor-product model specification}
\usage{
tp(first, second)
}
\arguments{
\item{first}{ngme_operator, left side of kronecker model (usually a temporal or iid model)}

\item{second}{ngme_operator, right side of kronecker model (ususally a temporal or spatial model)}
}
\value{
a list of specification of model
}
\description{
Given 2 operator (first and second), build a tensor-product operator based on K = K_first x K_second (here x is Kronecker product)
}
