% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/util.R
\name{get_noise_param}
\alias{get_noise_param}
\title{get the parameters of the noise}
\usage{
get_noise_param(noise, prefix)
}
\arguments{
\item{noise}{noise object}

\item{prefix}{prefix of the parameter}
}
\value{
a list with parameter name as key and parameter value as value
}
\description{
Get the parameters of the noise
}
\keyword{internal}
