% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/simulation.R
\name{simulate.ngme_noise}
\alias{simulate.ngme_noise}
\title{Simulate ngme noise object}
\usage{
\method{simulate}{ngme_noise}(object, nsim = 1, seed = NULL, h = NULL, ...)
}
\arguments{
\item{object}{ngme noise object}

\item{nsim}{number of simulations}

\item{seed}{seed}

\item{h}{should be of same length as nsim}

\item{...}{ignored}
}
\value{
data.frame (each col is a realization)
}
\description{
Simulate ngme noise object
}
