% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/util.R
\name{ngme_post_samples}
\alias{ngme_post_samples}
\title{posterior samples of different latent models}
\usage{
ngme_post_samples(ngme_object, model_name = 1, type = "W", replicate = 1)
}
\arguments{
\item{ngme_object}{ngme object}

\item{model_name}{name of the model, or index of the model}

\item{type}{type of samples, "W" or "V"}

\item{replicate}{which replicate}
}
\value{
a data.frame of posterior samples (mesh_size * n_post_samples)
}
\description{
Extract the posterior samples of different latent models
}
