% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ngme.R
\name{summary.ngme}
\alias{summary.ngme}
\title{Summary of ngme fit result}
\usage{
\method{summary}{ngme}(object, name = NULL, ...)
}
\arguments{
\item{object}{an object of class \code{ngme}}

\item{name}{name of the latent model to be summarized (if NULL, will print all)}

\item{...}{other arguments}
}
\value{
a list of summary
}
\description{
Summary of ngme fit result
}
