% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/test.ngme.R
\name{test_ngme}
\alias{test_ngme}
\title{Test ngme function}
\usage{
test_ngme(
  model,
  n_obs_per_rep,
  n_replicate = 1,
  control_opt = NULL,
  numer_grad = TRUE,
  f_noise = noise_nig(mu = -2, sigma = 1.5, nu = 0.5),
  n_gibbs_samples = 5,
  family = "normal",
  max.n = 1000,
  debug = FALSE,
  debug_f = FALSE,
  start = NULL,
  fix_theta_K = FALSE,
  fix_theta_mu = FALSE,
  fix_theta_sigma = FALSE,
  fix_theta_nu = FALSE,
  seed = Sys.time()
)
}
\arguments{
\item{model}{model name}

\item{n_obs_per_rep}{number of observation per replicate}

\item{n_replicate}{number of replicate}

\item{control_opt}{control options for optimization, see \code{\link{control_opt}}}

\item{numer_grad}{numerical gradient}

\item{f_noise}{noise function}

\item{n_gibbs_samples}{number of gibbs samples}

\item{family}{family of noise}

\item{max.n}{maximum number for building mesh}

\item{debug}{debug mode}

\item{debug_f}{debug mode for latent process}

\item{start}{start value for optimization}

\item{fix_theta_K}{fix theta_K}

\item{fix_theta_mu}{fix theta_mu}

\item{fix_theta_sigma}{fix theta_sigma}

\item{fix_theta_nu}{fix nu}

\item{seed}{seed}
}
\description{
Test ngme function for different models
}
\keyword{internal}
