% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/prior.R
\name{prior_half_cauchy}
\alias{prior_half_cauchy}
\title{Prior Half-Cauchy}
\usage{
prior_half_cauchy(scale = 1, target = "coef")
}
\arguments{
\item{scale}{half-Cauchy scale}

\item{target}{apply prior on coefficient scale (`"coef"`) or field scale (`"field"`)}
}
\value{
prior specification
}
\description{
Prior Half-Cauchy
}
