% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/util.R
\name{ngme_cov_matrix}
\alias{ngme_cov_matrix}
\title{variance of the data or the latent field}
\usage{
ngme_cov_matrix(ngme_object, model_name = "data", replicate = 1)
}
\arguments{
\item{ngme_object}{ngme_model}

\item{model_name}{if model_name = "data", then return the covariance matrix of the data (without measurement noise)
if the model_name is the name or index of the latent, then return the covariance matrix of the latent field}

\item{replicate}{which replicate (default = 1)}
}
\value{
a data.frame of posterior samples (mesh_size * n_post_samples)
}
\description{
Compute the variance of the data or the latent field
}
