% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ngme.constant.R
\name{ngme_noise_types}
\alias{ngme_noise_types}
\title{Show ngme noise types}
\usage{
ngme_noise_types()
}
\value{
available types for noise
}
\description{
Show ngme noise types
}
