% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/noise.R
\name{ngme_noise}
\alias{ngme_noise}
\alias{noise_nig}
\alias{noise_normal}
\alias{noise_t}
\alias{noise_skew_t}
\alias{noise_gal}
\alias{noise_normal_nig}
\title{ngme noise specification}
\usage{
ngme_noise(
  noise_type,
  mu = 0,
  sigma = 1,
  nu = 1,
  B_mu = NULL,
  theta_mu = NULL,
  B_sigma = NULL,
  theta_sigma = NULL,
  B_nu = NULL,
  theta_nu = NULL,
  theta_sigma_normal = NULL,
  B_sigma_normal = NULL,
  fix_theta_mu = FALSE,
  fix_theta_sigma = FALSE,
  fix_rho = FALSE,
  fix_theta_sigma_normal = FALSE,
  fix_theta_nu = FALSE,
  V = NULL,
  fix_V = FALSE,
  single_V = FALSE,
  share_V = FALSE,
  corr_measurement = FALSE,
  index_corr = NULL,
  map_corr = NULL,
  nu_lower_bound = 0,
  rho = double(0),
  prior = NULL,
  ...
)

noise_normal(
  sigma = NULL,
  theta_sigma = NULL,
  B_sigma = matrix(1),
  corr_measurement = FALSE,
  index_corr = NULL,
  ...
)

noise_nig(
  mu = NULL,
  sigma = NULL,
  nu = NULL,
  V = NULL,
  theta_mu = NULL,
  theta_sigma = NULL,
  theta_nu = NULL,
  nu_lower_bound = 0,
  B_mu = matrix(1),
  B_sigma = matrix(1),
  B_nu = matrix(1),
  corr_measurement = FALSE,
  index_corr = NULL,
  ...
)

noise_gal(
  mu = NULL,
  sigma = NULL,
  nu = NULL,
  V = NULL,
  theta_mu = NULL,
  theta_sigma = NULL,
  theta_nu = NULL,
  nu_lower_bound = 0.01,
  B_mu = matrix(1),
  B_sigma = matrix(1),
  B_nu = matrix(1),
  corr_measurement = FALSE,
  index_corr = NULL,
  ...
)

noise_skew_t(
  mu = NULL,
  sigma = NULL,
  nu = NULL,
  theta_mu = NULL,
  theta_sigma = NULL,
  theta_nu = NULL,
  nu_lower_bound = 0.01,
  B_mu = matrix(1),
  B_sigma = matrix(1),
  B_nu = matrix(1),
  corr_measurement = FALSE,
  index_corr = NULL,
  ...
)

noise_t(
  nu = NULL,
  theta_nu = NULL,
  nu_lower_bound = 0,
  B_nu = matrix(1),
  corr_measurement = FALSE,
  index_corr = NULL,
  ...
)

noise_normal_nig(
  sigma_normal = NULL,
  mu = NULL,
  sigma_nig = NULL,
  nu = NULL,
  V = NULL,
  theta_mu = NULL,
  theta_sigma_nig = NULL,
  theta_sigma_normal = NULL,
  theta_nu = NULL,
  B_mu = matrix(1),
  B_sigma_nig = matrix(1),
  B_sigma_normal = matrix(1),
  B_nu = matrix(1),
  corr_measurement = FALSE,
  index_corr = NULL,
  ...
)
}
\arguments{
\item{noise_type}{type of noise, "normal", "nig", "gal", "t", "skew_t"}

\item{mu}{specify the NIG noise parameter mu, see \code{?nig}}

\item{sigma}{specify the noise parameter sigma, see \code{?nig}}

\item{nu}{specify the noise parameter nu, see \code{?nig} and \code{?gal}}

\item{B_mu}{Basis matrix for mu (if non-stationary)}

\item{theta_mu}{specify a non-stationary noise using theta_mu}

\item{B_sigma}{Basis matrix for sigma (if non-stationary)}

\item{theta_sigma}{specify a non-stationary noise using theta_sigma}

\item{B_nu}{Basis matrix for nu (if non-stationary)}

\item{theta_nu}{specify a non-stationary noise using theta_nu}

\item{theta_sigma_normal}{for normal nosie with nig noise sharing same parameter}

\item{B_sigma_normal}{for normal nosie with nig noise sharing same parameter}

\item{fix_theta_mu}{fix the parameter of theta_mu}

\item{fix_theta_sigma}{fix the parameter of theta_sigma, can be a single
logical value or a vector of logical values with length equal to
length(theta_sigma)}

\item{fix_rho}{fix the parameter of rho}

\item{fix_theta_sigma_normal}{fix the parameter of sigma_normal, used in noise_normal_nig()}

\item{fix_theta_nu}{fix the parameter of nu}

\item{V}{start value for V}

\item{fix_V}{fix the sampling of V
gives Y|W ~ N(mean * sigma, sigma^2)}

\item{single_V}{TRUE if V is a single number}

\item{share_V}{used only for bivariate model}

\item{corr_measurement}{TRUE if we use correlated measurement noise}

\item{index_corr}{used when corr_measurement=TRUE, indicate which observation has correlation}

\item{map_corr}{1d, 2d, or formula, used when corr_measurement=TRUE, specify use which covariate to infer the index_corr.}

\item{nu_lower_bound}{specify the lower bound of parameter nu; effective
parametrization is \eqn{nu = nu_lower_bound + \exp(B_\nu \theta_\nu)} so
\code{theta_nu} remains unconstrained. Default 0.}

\item{rho}{used when corr_measurement=TRUE, starting point for correlation}

\item{prior}{prior specification created by \code{priors(...)}. Supported
keys are \code{mu}, \code{sigma}, and \code{nu}.}

\item{...}{additional arguments}

\item{sigma_normal}{for normal nosie with nig noise sharing same parameter}

\item{sigma_nig}{similar to sigma_normal}

\item{theta_sigma_nig}{similar to theta_sigma_normal}

\item{B_sigma_nig}{similar to B_sigma_nig}
}
\value{
a list of specification of noise
}
\description{
Function for specifying ngme noise.
Please use \code{noise_nig} and \code{noise_normal} for simpler usage.
Use \code{ngme_noise_types()} to check all the available types.
}
\details{
The parameterization is given in \code{?nig} and \code{?gal}. Moreover,
for specifying non-stationary mu and sigma, nu
\deqn{\mu = B_{\mu} \theta_{\mu},} and
\deqn{\sigma = \exp (B_{\sigma} \theta_{\sigma}),}
\deqn{\nu = \nu_{\mathrm{lower}} + \exp (B_{\nu} \theta_{\nu}).}
}
\examples{
noise_normal(sigma = 2)
noise_nig(mu = 1, sigma = 2, nu = 1)
noise_gal(mu = 1, sigma = 2, nu = 1)
noise_skew_t(mu = 0, sigma = 1, nu = 5)
noise_t(nu = 5)
}
