% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/info_datasets.R
\docType{data}
\name{cienaga.border}
\alias{cienaga.border}
\title{The x y location of the border of the swamp of Cienaga Grande in Santa Marta, Colombia}
\format{
A data frame with 472 rows and 2 columns.
\describe{
  \item{East, North}{location}
}
}
\source{
..
}
\usage{
cienaga.border
}
\description{
The data is of dimension 472 * 2. It contains the x and y coordinates of the border of the swamp.
}
\keyword{datasets}
