% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/util.R
\name{mean_list}
\alias{mean_list}
\title{taking mean over a list of nested lists}
\usage{
mean_list(lls, weights = NULL)
}
\arguments{
\item{lls}{a list}

\item{weights}{weights of each list}
}
\value{
a list of nested lists
}
\description{
taking mean over a list of nested lists
}
\examples{
ls <- list(
  list(a = 1, b = 2, t = "nig", ll = list(a = 1, b = 2, w = "ab")),
  list(a = 3, b = 5, t = "nig", ll = list(a = 1, b = 6, w = "ab")),
  list(a = 5, b = 5, t = "nig", ll = list(a = 4, b = 2, w = "ab"))
)
mean_list(ls)
}
