% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/loglik.R
\name{compute_log_like}
\alias{compute_log_like}
\title{Compute Gaussian log-likelihood}
\usage{
compute_log_like(ngme_model)
}
\arguments{
\item{ngme_model}{An object created by `ngme()`.}
}
\value{
A numeric scalar. Returns `0` if any replicate is non-Gaussian.
}
\description{
Creates the underlying C++ block models and evaluates their Gaussian log-likelihood when applicable.
}
