% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/optimizer.R
\name{poly_decay}
\alias{poly_decay}
\title{Polynomial schedule helper}
\usage{
poly_decay(alpha = 0.501, t0 = 1, burnin_iter = 0)
}
\arguments{
\item{alpha}{polynomial exponent in \eqn{(1/2,1)}.}

\item{t0}{non-negative schedule offset.}

\item{burnin_iter}{non-negative integer. Initial iterations without
polynomial schedule scaling.}
}
\value{
a \code{stepsize_control()} object.
}
\description{
Polynomial schedule helper
}
\details{
Convenience helper that enables polynomial schedule and disables
checkpoint-based decay.
}
