% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/util.R
\name{ngme_as_sparse}
\alias{ngme_as_sparse}
\title{Convert sparse matrix into sparse dgCMatrix}
\usage{
ngme_as_sparse(G)
}
\arguments{
\item{G}{matrix}
}
\value{
sparse dgCMatrix
}
\description{
Convert sparse matrix into sparse dgCMatrix
}
