% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/util.R
\name{ngme_update}
\alias{ngme_update}
\title{Check whether a newer stable version of ngme2 is available}
\usage{
ngme_update()
}
\value{
Invisibly returns a list with the local version, remote version,
  repository URL, and a logical \code{update_available} flag.
}
\description{
This function checks the package repository for the latest available ngme2
version. It does not install or update packages.
}
