% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/operator.R
\name{print.ngme_operator}
\alias{print.ngme_operator}
\title{Print ngme operator}
\usage{
\method{print}{ngme_operator}(x, padding = 0, prefix = "Model type", ...)
}
\arguments{
\item{x}{ngme operator object}

\item{padding}{number of white space padding in front}

\item{prefix}{prefix string}

\item{...}{...}
}
\value{
a list (operator specifications)
}
\description{
Print ngme operator
}
