% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/optimizer.R
\name{precond_sgd}
\alias{precond_sgd}
\title{Preconditioner SGD optimization}
\usage{
precond_sgd(stepsize = 1, numerical_eps = 1e-05)
}
\arguments{
\item{stepsize}{stepsize for SGD}

\item{numerical_eps}{numerical, the gap used for estimate preconditioner, default is 1e-5}
}
\value{
a list of control variables for optimization
(used in \code{control_opt} function)
}
\description{
Preconditioner SGD optimization
}
\details{
Preconditioner SGD is using fisher information matrix as preconditioner (natural gradient descent).
}
