% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.ngme.R
\name{compare_noise_kld}
\alias{compare_noise_kld}
\title{Compare noise objects using Kullback-Leibler divergence}
\usage{
compare_noise_kld(x = NULL, ..., xlim = c(-10, 10), n_points = 1000)
}
\arguments{
\item{x}{first noise object (reference)}

\item{...}{additional noise objects to compare, and optional parameters}

\item{xlim}{x-axis range for evaluation (default: c(-10, 10))}

\item{n_points}{number of evaluation points (default: 1000)}
}
\value{
named vector of KLD values
}
\description{
This function compares multiple noise objects by calculating the KLD
of each noise against the first noise object (reference).
}
\examples{
n1 <- noise_nig(mu = 0, sigma = 1, nu = 1)
n2 <- noise_nig(mu = 0.5, sigma = 1.2, nu = 0.8)
compare_noise_kld(n1, method2 = n2)
}
