% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/util.R
\name{get_trajectories}
\alias{get_trajectories}
\title{get the trajectories of parameters of the model}
\usage{
get_trajectories(ngme_object, model_name)
}
\arguments{
\item{ngme_object}{ngme object}

\item{model_name}{name of the model, if NULL, return the trajectories of the parameters of the measurement noise and fixed effects}
}
\value{
a list of trajectories, each element is a matrix of trajectories (n_samples * n_trajectories)
}
\description{
Get the trajectories of the parameters of the model
}
