% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/models.R
\name{iid}
\alias{iid}
\title{ngme iid model specification}
\usage{
iid(mesh = NULL)
}
\arguments{
\item{mesh}{integer or factor, index to build the mesh}
}
\value{
ngme_operator object
}
\description{
ngme iid model specification
}
