% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/noise.R
\name{merge_noise}
\alias{merge_noise}
\title{Merge 2 noise into 1 noise}
\usage{
merge_noise(noise1, noise2)
}
\arguments{
\item{noise1}{noise 1}

\item{noise2}{noise 2}
}
\value{
merged noise
}
\description{
Merge 2 noise into 1 noise
}
