% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/util.R
\name{ngme_make_mesh_repls}
\alias{ngme_make_mesh_repls}
\title{ngme make mesh for different replicates}
\usage{
ngme_make_mesh_repls(data, map, replicate, mesh_type = "regular")
}
\arguments{
\item{data}{provide the data.frame}

\item{map}{provide the map to make mesh, i.g. ~x+y, x and y will be extracted from data to make a 2d mesh}

\item{replicate}{provide the replicate information, i.g. ~id}

\item{mesh_type}{type of mesh, "regular" means use all the point from same replicate to make a mesh}
}
\value{
a list of mesh of length of different replicates
}
\description{
Make different mesh for different replicates
}
