% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ngme.R
\name{print.ngme}
\alias{print.ngme}
\title{Print an ngme model}
\usage{
\method{print}{ngme}(x, ...)
}
\arguments{
\item{x}{ngme model object}

\item{...}{...}
}
\value{
Invisibly returns \code{x}, an \code{ngme} object. The method is
  called for its side effect of printing a model summary.
}
\description{
Print an ngme model
}
