% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.ngme.R
\name{print.noise_kld_comparison}
\alias{print.noise_kld_comparison}
\title{Print method for noise_kld_comparison}
\usage{
\method{print}{noise_kld_comparison}(x, ...)
}
\arguments{
\item{x}{noise_kld_comparison object}

\item{...}{additional arguments}
}
\value{
Invisibly returns \code{x}, a \code{noise_kld_comparison} object. The
  method is called for its side effect of printing the reference noise object,
  the Kullback-Leibler divergence values for each comparison noise, and the
  closest comparison.
}
\description{
Print method for noise_kld_comparison
}
