% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/optimizer.R
\name{rmsprop}
\alias{rmsprop}
\title{Root Mean Square Propagation (RMSProp) SGD optimization}
\usage{
rmsprop(stepsize = 0.05, beta1 = 0.9, epsilon = 1e-08)
}
\arguments{
\item{stepsize}{stepsize for SGD}

\item{beta1}{beta1 for momentum}

\item{epsilon}{epsilon for numerical stability}
}
\value{
a list of control variables for optimization
(used in \code{control_opt} function)
}
\description{
Root Mean Square Propagation (RMSProp) SGD optimization
}
\details{
The update rule for RMSProp is:
\deqn{v_t = \beta_1 v_{t-1} + (1 - \beta_1) g_t^2}
\deqn{x_{t+1} = x_t - \text{stepsize} * \frac{g_t}{\sqrt{v_t} + \epsilon}}
}
